@@ -4,11 +4,26 @@ from __future__ import unicode_literals
 import mimetypes
 import os
 import re
+import sys
 from tempfile import NamedTemporaryFile
+from unicodedata import normalize
+
+try:
+    from urllib.parse import urlparse
+except ImportError:
+    from urlparse import urlparse
 
 import requests
 from twitter import TwitterError
 
+if sys.version_info < (3,):
+    range = xrange
+
+CHAR_RANGES = [
+    range(0, 4351),
+    range(8192, 8205),
+    range(8208, 8223),
+    range(8242, 8247)]
 
 TLDS = [
     "ac", "ad", "ae", "af", "ag", "ai", "al", "am", "an", "ao", "aq", "ar",
@@ -143,7 +158,7 @@ URL_REGEXP = re.compile((
     r'('
     r'^(?!(https?://|www\.)?\.|ftps?://|([0-9]+\.){{1,3}}\d+)'  # exclude urls that start with "."
     r'(?:https?://|www\.)*^(?!.*@)(?:[\w+-_]+[.])'              # beginning of url
-    r'(?:{0}\b|'                                                # all tlds
+    r'(?:{0}\b'                                                # all tlds
     r'(?:[:0-9]))'                                              # port numbers & close off TLDs
     r'(?:[\w+\/]?[a-z0-9!\*\'\(\);:&=\+\$/%#\[\]\-_\.,~?])*'    # path/query params
     r')').format(r'\b|'.join(TLDS)), re.U | re.I | re.X)
@@ -166,7 +181,11 @@ def calc_expected_status_length(status, short_url_length=23):
         if is_url(word):
             status_length += short_url_length
         else:
-            status_length += len(word)
+            for character in word:
+                if any([ord(normalize("NFC", character)) in char_range for char_range in CHAR_RANGES]):
+                    status_length += 1
+                else:
+                    status_length += 2
     status_length += len(re.findall(r'\s', status))
     return status_length
 
@@ -215,7 +234,7 @@ def parse_media_file(passed_media):
     if not hasattr(passed_media, 'read'):
         if passed_media.startswith('http'):
             data_file = http_to_file(passed_media)
-            filename = os.path.basename(passed_media)
+            filename = os.path.basename(urlparse(passed_media).path)
         else:
             data_file = open(os.path.realpath(passed_media), 'rb')
             filename = os.path.basename(passed_media)
